#include "pch.h"
#include "ScatterGather.h"

// . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .

size_t
getScatterGatherSize (
	const MemBlock* blockArray,
	size_t blockCount
	)
{
	const MemBlock* block = blockArray;
	const MemBlock* blockEnd = block + blockCount;

	size_t size = 0;

	for (; block < blockEnd; block++)
		size += block->m_size;

	return size;
}

static
inline
ssize_t
copyMemBlock (
	void* dst,
	const void* src,
	size_t size,
	uint flags
	)
{
	int result;

	if (!(flags & MemBlockFlag_UserBuffer))
	{
		memcpy (dst, src, size);
	}
	else
	{
		result = copy_from_user (dst, src, size);
		if (result != 0)
			return -EFAULT;
	}

	return 0;
}

ssize_t
copyScatterGather_u (
	void* p, // must be big enough
	const MemBlock* blockArray,
	size_t blockCount
	)
{
	int result;
	const MemBlock* block = blockArray;
	const MemBlock* blockEnd = block + blockCount;
	char* dst = p;

	for (; block < blockEnd; block++)
	{
		result = copyMemBlock (dst, block->m_p, block->m_size, block->m_flags);
		if (result != 0)
			return result;

		dst += block->m_size;
	}

	return dst - (char*) p;
}

ssize_t
copyScatterGather_s (
	void* p,
	size_t size,
	const MemBlock* blockArray,
	size_t blockCount
	)
{
	int result;
	const MemBlock* block = blockArray;
	const MemBlock* blockEnd = block + blockCount;
	char* dst = p;
	char* dstEnd = dst + size;

	for (; block < blockEnd; block++)
	{
		size_t leftover = dstEnd - dst;
		if (leftover < block->m_size)
		{
			result = copyMemBlock (dst, block->m_p, leftover, block->m_flags);
			if (result != 0)
				return result;

			dst += leftover;
			break;
		}

		result = copyMemBlock (dst, block->m_p, block->m_size, block->m_flags);
		if (result != 0)
			return result;

		dst += block->m_size;
	}

	return dst - (char*) p;
}

//..............................................................................
