#include "pch.h"
#include "lkmUtils.h"

//..............................................................................

ulong
disablePreemptionAndWriteProtection (void)
{
    ulong cr0;

    preempt_disable(); // barrier is created

    cr0 = read_cr0 ();
    write_cr0 (cr0 & ~X86_CR0_WP);
    return cr0;
}

void
restoreWriteProtectionAndPreemption (ulong cr0)
{
    write_cr0 (cr0);

    preempt_enable (); // barrier is created
}

char*
createPathString (const struct path* path)
{
	char buffer [128]; // more than enough
	char* src;
	char* dst;
	size_t size;

	src = d_path (path, buffer, sizeof (buffer) - 1);
	if (IS_ERR (src))
		return src;

	buffer [sizeof (buffer) - 1] = 0;
	size = strlen (src) + 1;
	dst = kmalloc (size, GFP_KERNEL);
	if (!dst)
		return ERR_PTR (-ENOMEM);

	memcpy (dst, src, size);
	return dst;
}

uint64_t
getTimestamp (void)
{
	enum
	{
		// epoch difference between Unix time (1 Jan 1970 00:00) and Windows time (1 Jan 1601 00:00)
		EpochDiff = 11644473600LL
	};

	struct timespec tspec;
	ktime_get_real_ts (&tspec);

	return (uint64_t) (tspec.tv_sec + EpochDiff) * 10000000 + tspec.tv_nsec / 100;
}

struct module*
getOwnerModule (struct file* filp)
{
	if (filp->f_op && filp->f_op->owner)
		return filp->f_op->owner;

	if (!filp->f_inode) // bummer, nowhere to go from here. but wait, is it even possible?
		return NULL;

	if (!S_ISCHR (filp->f_inode->i_mode))
	{
		printk (KERN_WARNING "tdevmon: couldn't find owner module of non-char device (%x)\n", filp->f_inode->i_mode);
		return NULL;
	}

	if (filp->f_inode->i_cdev &&
		filp->f_inode->i_cdev->owner)
		return filp->f_inode->i_cdev->owner;

	printk (
		KERN_WARNING
		"tdevmon: couldn't find owner module (cdev: %p, cdev->owner: %p); "
		"TODO: lookup via f_inode->i_rdev (%x))\n",
		filp->f_inode->i_cdev,
		filp->f_inode->i_cdev->owner,
		filp->f_inode->i_rdev
		);

	return NULL;
}

//..............................................................................
